famQBL = function(data, num_haplotypes, file1 = "haplore_input.dat", file2 = "haplore_output.dat", start.beta = 0.1, lambda = 1, D = 0, burn.in = 20000, num.it = 70000, seed = NULL, a = 20, b = 20, e = 0.1)
{
  dyn.load("famQBL.so")
  library(kinship2)
  set.seed(1234)
  num_beta = num_haplotypes 
  
  ped.geno.pheno = data 
  ped.geno.pheno = ped.geno.pheno[order(ped.geno.pheno$V1),] 
  fbat_ped = ped.geno.pheno
  ped = ped.geno.pheno[,1:5] #not including the genotype column, needed for creating relationship matrix
  family = unique(ped$V1) 
  phenos = ped.geno.pheno[,dim(ped.geno.pheno)[2]]
  
  num_fam = length(family) #storing number of families
  tot_indiv = dim(ped)[1] # storing total individual in all families
  poly_inv = matrix(nrow=tot_indiv,ncol=tot_indiv,0)
  
  sum = 0
  for(i in 1:num_fam){
    ped.sub1 = ped[ped$V1==family[i],] 
    n1 = dim(ped.sub1)[1] 
    test1 = data.frame(id=ped.sub1$V2,dad=ped.sub1$V3,mom=ped.sub1$V4,sex=ped.sub1$V5) 
    tped1 <- with(test1, pedigree(id, dad, mom, sex)) 
    kin = kinship(tped1) 
    kin = 2*kin          
    kin_inv = solve(kin) 
    start = sum + 1
    end = sum + n1
    poly_inv[start:end,start:end] = kin_inv[1:n1,1:n1]
    sum = sum + n1
  }
  polygenic = rep(0,tot_indiv)
  affection = double(length = tot_indiv)
  
  lenp = length(phenos)
  lenm = length(phenos[phenos==0]) 
  miss = integer(length=lenm)
  j = 1
  for(i in 1:lenp){
    if(phenos[i]==0){
      miss[j] = i
      j = j + 1
    }
  }
  
  #standardizing the trait values using trait values of non-missing individuals
  nonMiss.trait = double(length=(lenp-lenm))
  j = 1
  for(i in 1:lenp){
    if(phenos[i]!=0){
      nonMiss.trait[j] = phenos[i]
      j = j + 1
    }
  }
  mean.trait = mean(nonMiss.trait)
  sd.trait = sd(nonMiss.trait)
  trait.std = (phenos - mean.trait)/sd.trait
  trait.std[miss] = 0
  
  #finding the traits of founders
  num.founder = length(ped$V3[ped$V3==0]) 
  miss.founder = subset(ped.geno.pheno,V3==0 & V16==0)
  num.noMiss.founder = num.founder - dim(miss.founder)[1]
  
  k = 1
  noMiss.trait.founder = double(length=num.noMiss.founder)
  for(i in 1:tot_indiv)
  {
    if((ped$V3[i] ==0 && ped$V4[i]==0) && phenos[i]>0)
    {
      noMiss.trait.founder[k] = phenos[i]
      k = k+1
    } 
  }
  mean.trait.founder = mean(noMiss.trait.founder)
  var.trait.founder = sd(noMiss.trait.founder)*sd(noMiss.trait.founder)
  
  beta.out<-numeric((num.it-burn.in)*num_beta)
  lambda.out<-numeric(num.it-burn.in)
  freq.out<-numeric((num.it-burn.in)*(num_beta))
  D.out<-numeric(num.it-burn.in)
  ssA.out<-numeric(num.it-burn.in)
  ssE.out<-numeric(num.it-burn.in)
  ped_matrix = data.matrix(ped)
  out = .C("mcmc", as.character(file1), as.character(file2), as.double(polygenic), as.double(poly_inv), as.integer(tot_indiv), as.double(start.beta), as.integer(num_fam), as.double(trait.std), as.double(mean.trait.founder), as.double(var.trait.founder), as.double(mean.trait), as.integer(num.it), as.integer(burn.in), beta.out=as.double(beta.out), freq.out=as.double(freq.out), lambda.out=as.double(lambda.out), D.out=as.double(D.out), ssA.out=as.double(ssA.out), ssE.out=as.double(ssE.out))
  
  out$beta.out = round(out$beta.out,6)
  out$freq.out = round(out$freq.out,6)
  out$lambda.out = round(out$lambda.out,6)
  out$D.out = round(out$D.out,6)
  out$ssA.out = round(out$ssA.out,6)
  out$ssE.out = round(out$ssE.out,6)
  
  beta.out = matrix(out$beta.out,nrow=num.it-burn.in,byrow=TRUE)
  freq.out = matrix(out$freq.out,nrow=num.it-burn.in,byrow=TRUE)
  
  ci.beta<-numeric(num_beta*2)
  post.mean.beta<-numeric(num_beta)
  
  k<-1
  for (i in 1:num_beta)
  {
    ci.beta[k]<-quantile(beta.out[,i], probs=0.025)
    ci.beta[k+1]<-quantile(beta.out[,i], probs=0.975)
    k<-k+2
    post.mean.beta[i]<- mean(beta.out[,i])
  }
  
  prob.alt<-numeric(num_beta)
  BF<-numeric(num_beta)
  
  for (i in 1:num_beta)
  {
    prob.alt[i]<-length(beta.out[,i][abs(beta.out[,i]) > e])/(num.it-burn.in)
  }
  
  for (i in 1:num_beta)
  {
    prior.prob<-(b/(e+b))^a
    if (prob.alt[i]<1) 
    {
      BF[i]<-(prob.alt[i]/(1-prob.alt[i]))/(prior.prob/(1-prior.prob))
      BF[i] = round(BF[i],4)
    } else
      BF[i]<-">100"
  }
  
  ci.beta<-matrix(ci.beta,nrow=num_beta, ncol=2, byrow=TRUE)
  cs = cbind(t(ci.beta[,1]),t(ci.beta[,2]))
  
  out1 = cbind(t(BF),t(post.mean.beta))
  #write.table(out1,file="famQBL_BF_beta.out",append=TRUE,quote=FALSE,row.names=FALSE,col.names=FALSE)
  #write.table(cs,file="famQBL_CredibleSet.out",append=TRUE,quote=FALSE,row.names=FALSE,col.names=FALSE)
  
  hap = read.table("hap-freq.txt",h=F)
  hap = hap[,-(1:3)]
  col = dim(hap)[2]
  hap[,col] = round(hap[,col],4)
  num_haplotypes = 6
  for(i in 1:num_haplotypes){
    for(j in 1:(col-1)){
      if(hap[i,j]==2) hap[i,j] = 0
    }
  }
  hap.cols = character(length = num_haplotypes)
  for(i in 1:num_haplotypes){
    hap.cols[i] = paste0("h",hap[i,1],hap[i,2],hap[i,3],hap[i,4],hap[i,5])
  }
  haplo.info = data.frame(hap.cols,hap[,col])
  names(haplo.info) = c("haplotype","freq")
  haplo.info = haplo.info[order(-haplo.info$freq),]
  file.remove("hap-freq.txt")
  file.out = data.frame(haplo.info,BF,round(post.mean.beta,4),round(ci.beta,4))
  names(file.out) = c("Haplotype","Freq","BF","Beta","Low_CS","Up_CS")
  write.csv(file.out,file="famQBL_analysis.csv",row.names = FALSE)
  print(paste0("Analysis DONE: Please see file famQBL_analysis.csv for output."))
}
